@@ -21,7 +21,6 @@ Adapted to BirdFont by Johan Mattsson 2015
 /*	Piecewise cubic fitting code	*/
 
 #include "GraphicsGems.h"
-#include <stdio.h>
 
 #ifdef MAC
 #include <malloc/malloc.h>
@@ -31,6 +30,8 @@ Adapted to BirdFont by Johan Mattsson 2015
 
 #include <math.h>
 #include <stdio.h>
+#include <glib.h>
+
 
 typedef Point2 *BezierCurve;
 
@@ -53,9 +54,25 @@ static	Vector2		V2SubII();
 
 #define MAXPOINTS	1000		/* The most points you can have */
 
-int simplified_path_buffer_size = 0;
-int simplified_path_size = 0;
-double* simplified_path = NULL;
+typedef struct {
+	int simplified_path_buffer_size ;
+	int simplified_path_size;
+	double* simplified_path;
+} Buffer;
+
+Buffer* gems_buffer_new (int buffer_size, double* simplified_path) {
+	Buffer* b = (Buffer*) malloc (sizeof (Buffer));
+	
+	b->simplified_path = simplified_path;
+	b->simplified_path_buffer_size = buffer_size;
+	b->simplified_path_size = 0;
+	
+	return b;
+}
+
+void gems_buffer_delete (Buffer* b) {
+	free (b);
+}
 
 /** Generates an a bezier path and returns the length of the output array. */ 
 void fit_bezier_curve_to_line (
@@ -78,11 +95,6 @@ void fit_bezier_curve_to_line (
 		fprintf (stderr, "No lines in fit_bezier_curve_to_line.");
 		return;
 	}
-
-	if (simplified_path != NULL) {
-		fprintf (stderr, "Path simplification is alreading running.");
-		return;	
-	}
 	
 	if (bezier_path == NULL) {
 		fprintf (stderr, "No destination for output buffer in fit_bezier_curve_to_line");
@@ -104,35 +116,36 @@ void fit_bezier_curve_to_line (
 		j += 2;
 	}
 	
-	simplified_path = malloc (8 * npoints * sizeof (double));
-	simplified_path_buffer_size = 8 * npoints;
-	simplified_path_size = 0;
- 
-	FitCurve(points, npoints, error);
+	int buffer_size = 8 * lines_size;
+	double* simplified_path = malloc (buffer_size * sizeof (double));
+	Buffer* buffer = gems_buffer_new (buffer_size, simplified_path);
+	
+	FitCurve(buffer, points, npoints, error);
 	
 	*bezier_path = simplified_path;
-	*bezier_path_size = simplified_path_size;
-	
-	simplified_path = NULL;
+	*bezier_path_size = buffer->simplified_path_size;
 	
+	gems_buffer_delete (buffer);
 	free (points);
 }
 
-DrawBezierCurve(int n, Point2* curve)
+void DrawBezierCurve(Buffer* buffer, int n, Point2* curve)
 {
 	int i;
+	double* simplified_path;
 
-	if (simplified_path_size + 8 > simplified_path_buffer_size) {
-		fprintf (stderr, "The bezier buffer is full (%d).\n", simplified_path_buffer_size);
+	if (buffer->simplified_path_size + 8 > buffer->simplified_path_buffer_size) {
+		g_warning ("The bezier buffer is full (%d).\n", buffer->simplified_path_buffer_size);
 		return;
 	}
 
 	if (n != 3) {
-		fprintf (stderr, "Expecting three points\n");
+		g_warning ("Expecting three points\n");
 		return;
 	}
 	
-	i = simplified_path_size;
+	i = buffer->simplified_path_size;
+	simplified_path = buffer->simplified_path;
 
 	simplified_path[i + 0] = curve[0].x;
 	simplified_path[i + 1] = curve[0].y;
@@ -143,14 +156,15 @@ DrawBezierCurve(int n, Point2* curve)
 	simplified_path[i + 6] = curve[3].x;
 	simplified_path[i + 7] = curve[3].y;
 
-	simplified_path_size = i + 8;
+	buffer->simplified_path_size = i + 8;
 }
 
 /*
  *  FitCurve :
  *  	Fit a Bezier curve to a set of digitized points 
  */
-void FitCurve(d, nPts, error)
+void FitCurve(buffer, d, nPts, error)
+    Buffer* buffer;
     Point2	*d;			/*  Array of digitized points	*/
     int		nPts;		/*  Number of digitized points	*/
     double	error;		/*  User-defined error squared	*/
@@ -159,18 +173,20 @@ void FitCurve(d, nPts, error)
 
     tHat1 = ComputeLeftTangent(d, 0);
     tHat2 = ComputeRightTangent(d, nPts - 1);
-    FitCubic(d, 0, nPts - 1, tHat1, tHat2, error);
+    FitCubic(buffer, d, 0, nPts - 1, tHat1, tHat2, error, 0);
 }
 
 /*
  *  FitCubic :
  *  	Fit a Bezier curve to a (sub)set of digitized points
  */
-static void FitCubic(d, first, last, tHat1, tHat2, error)
+static void FitCubic(buffer, d, first, last, tHat1, tHat2, error, iterations)
+    Buffer* buffer;
     Point2	*d;			/*  Array of digitized points */
     int		first, last;	/* Indices of first and last pts in region */
     Vector2	tHat1, tHat2;	/* Unit tangent vectors at endpoints */
     double	error;		/*  User-defined error squared	   */
+    int iterations;
 {
     BezierCurve	bezCurve; /*Control points of fitted Bezier curve*/
     double	*u;		/*  Parameter values for point  */
@@ -183,9 +199,20 @@ static void FitCubic(d, first, last, tHat1, tHat2, error)
     Vector2	tHatCenter;   	/* Unit tangent vector at splitPoint */
     int		i;		
 
+	if (iterations > 2000) {
+		g_warning("Too many iterations.");
+		return;
+	}
+	iterations++;
+
     iterationError = error * error;
     nPts = last - first + 1;
 
+	if (nPts <= 1) {
+		g_warning("nPts <= 1");
+		return;
+	}
+	
     /*  Use heuristic if region only has two points in it */
     if (nPts == 2) {
 	    double dist = V2DistanceBetween2Points(&d[last], &d[first]) / 3.0;
@@ -195,25 +222,28 @@ static void FitCubic(d, first, last, tHat1, tHat2, error)
 		bezCurve[3] = d[last];
 		V2Add(&bezCurve[0], V2Scale(&tHat1, dist), &bezCurve[1]);
 		V2Add(&bezCurve[3], V2Scale(&tHat2, dist), &bezCurve[2]);
-		DrawBezierCurve(3, bezCurve);
+		DrawBezierCurve(buffer, 3, bezCurve);
 		free((void *)bezCurve);
 		return;
     }
 
     /*  Parameterize points, and attempt to fit curve */
     u = ChordLengthParameterize(d, first, last);
+    if (u == NULL) {
+		return;
+	}
+    
     bezCurve = GenerateBezier(d, first, last, u, tHat1, tHat2);
 
     /*  Find max deviation of points to fitted curve */
     maxError = ComputeMaxError(d, first, last, bezCurve, u, &splitPoint);
     if (maxError < error) {
-		DrawBezierCurve(3, bezCurve);
+		DrawBezierCurve(buffer, 3, bezCurve);
 		free((void *)u);
 		free((void *)bezCurve);
 		return;
     }
 
-
     /*  If error not too large, try some reparameterization  */
     /*  and iteration */
     if (maxError < iterationError) {
@@ -224,7 +254,7 @@ static void FitCubic(d, first, last, tHat1, tHat2, error)
 	    	maxError = ComputeMaxError(d, first, last,
 				       bezCurve, uPrime, &splitPoint);
 	    	if (maxError < error) {
-			DrawBezierCurve(3, bezCurve);
+			DrawBezierCurve(buffer, 3, bezCurve);
 			free((void *)u);
 			free((void *)bezCurve);
 			free((void *)uPrime);
@@ -239,9 +269,9 @@ static void FitCubic(d, first, last, tHat1, tHat2, error)
     free((void *)u);
     free((void *)bezCurve);
     tHatCenter = ComputeCenterTangent(d, splitPoint);
-    FitCubic(d, first, splitPoint, tHat1, tHatCenter, error);
+    FitCubic(buffer, d, first, splitPoint, tHat1, tHatCenter, error, iterations);
     V2Negate(&tHatCenter);
-    FitCubic(d, splitPoint, last, tHatCenter, tHat2, error);
+    FitCubic(buffer, d, splitPoint, last, tHatCenter, tHat2, error, iterations);
 }
 
 
@@ -544,7 +574,17 @@ static double *ChordLengthParameterize(d, first, last)
     int		i;	
     double	*u;			/*  Parameterization		*/
 
+	if (last-first+1 <= 0) {
+		g_warning("No array.");
+		return NULL;
+	}
+
     u = (double *)malloc((unsigned)(last-first+1) * sizeof(double));
+	
+	if (u == NULL) {
+		g_warning("Can't allocate array in ChordLengthParameterize");
+		return NULL;
+	}
 
     u[0] = 0.0;
     for (i = first+1; i <= last; i++) {
